@@ -6,11 +6,7 @@ function get_latest_git_tag {
   git describe --tags | grep -Eo '[0-9]+\.[0-9]+\.[0-9]+'
 }
 
-VERSION="${1:-`get_latest_git_tag`}"
-
-go fmt ./cmd/... ./pkg/... ./test/...
-go mod vendor
-go mod tidy
+VERSION="${1:-$(get_latest_git_tag)}"
 
 # makes builds reproducible
 export CGO_ENABLED=0
